--------------------------------------------------------------------
-- Project : SERIAL to WB_master interface
-- Author : Sebastin Fernndez
-- Date : 27/08/2008
-- File : serie2wbm.vhd
-- Design : Serie to wishbone Master 
--------------------------------------------------------------------
-- Description : Modulo Serie 8N1 a WB Master
-- El WM Master realiza ciclos de Lectura o Escritura simple a direccin 
-- contenida en registro interno
--
-- Protocolo de comunicacin:
-- * Seteo de direccin en registro interno: 
--    1. Enviar byte X"01"
--    2. Enviar byte con direccin a cargar en registro
-- * Ciclo de escritura WB
--    1. Enviar byte X"02"
--    2. Enviar byte con dato a escribir en esclavo WB
-- * Ciclo de lectura WB
--    1. Enviar byte X"04"
--    2. una vez recibido el dato por parte del maestro WB, es enviado en forma serie
--
-- Entidades que componen el diseo:
-- * serie2wbm
--    * uart
--       * uart_rx
--       * uart_tx
--    * wb_master   
--
-- Generics: BAUD_RATE_16x_COUNTER_REFERENCE
--    * Para Fclk=25.175MHz y 9600bps
--    * BAUD_RATE_16x_COUNTER_REFERENCE = F_CLK_I/(BPS * 15) -1 = 25175000 / (9600 * 15) -1 = 174
--------------------------------------------------------------------
-- $Log$
--------------------------------------------------------------------

---*******************************************************---
---*******************************************************---
---************  SERIE2WBM *******************************---
---*******************************************************---
---*******************************************************---

-----------------------------------------------------------
----- package PK_SERIE2WBM
-----------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

package PK_SERIE2WBM is
  component serie2wbm is
    generic(
      BAUD_RATE_16x_COUNTER_REFERENCE : integer
      );
    port(
      CLK_I   : in  std_logic;
      RESET_I : in  std_logic;
      --- RS232 signals
      TX_O    : out std_logic;
      RX_I    : in  std_logic;

      -- WB Interface
      CYC_O : out std_logic;
      CTI_O : out std_logic_vector(2 downto 0);
      STB_O : out std_logic;
      ACK_I : in  std_logic;
      WE_O  : out std_logic;
      ADR_O : out std_logic_vector(7 downto 0);
      DAT_I : in  std_logic_vector(7 downto 0);
      DAT_O : out std_logic_vector(7 downto 0)
      );
  end component;
end package;

-----------------------------------------------------------
----- ENTITY SERIE2WBM
-----------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
library work;
use work.PK_UART.all;
use work.PK_WB_MASTER.all;


entity serie2wbm is
  generic(
    BAUD_RATE_16x_COUNTER_REFERENCE : integer
    );
  port(
    CLK_I   : in  std_logic;
    RESET_I : in  std_logic;
    --- RS232 signals
    TX_O    : out std_logic;
    RX_I    : in  std_logic;

    -- WB Interface
    CYC_O : out std_logic;
    CTI_O : out std_logic_vector(2 downto 0);
    STB_O : out std_logic;
    ACK_I : in  std_logic;
    WE_O  : out std_logic;
    ADR_O : out std_logic_vector(7 downto 0);
    DAT_I : in  std_logic_vector(7 downto 0);
    DAT_O : out std_logic_vector(7 downto 0)
    );
end serie2wbm;


architecture struct of serie2wbm is

---- signals ----
  signal data_out                     : std_logic_vector(7 downto 0);
  signal new_received_data            : std_logic;
  signal new_received_data_read_pulse : std_logic;
  signal data_in                      : std_logic_vector(7 downto 0);
  signal ready_to_trasmit             : std_logic;
  signal new_data_to_trasmit          : std_logic;
  

begin

  uart_instance : uart
    generic map(
      BAUD_RATE_16x_COUNTER_REFERENCE => BAUD_RATE_16x_COUNTER_REFERENCE
      )
    port map(
      CLK_I                        => CLK_I,
      RESET_I                      => RESET_I,
      TX_O                         => TX_O,
      RX_I                         => RX_I,
      data_out                     => data_out,
      new_received_data            => new_received_data,
      new_received_data_read_pulse => new_received_data_read_pulse,
      data_in                      => data_in,
      ready_to_trasmit             => ready_to_trasmit,
      new_data_to_trasmit          => new_data_to_trasmit
      );

  wb_master_instance : wb_master
    port map(
      CLK_I   => CLK_I,
      RESET_I => RESET_I,

      CYC_O => CYC_O,
      CTI_O => CTI_O,
      STB_O => STB_O,
      ACK_I => ACK_I,
      WE_O  => WE_O ,
      ADR_O => ADR_O,
      DAT_I => DAT_I,
      DAT_O => DAT_O,

      uart_data_out                        => data_out,
      uart_new_received_data               => new_received_data,
      to_uart_new_received_data_read_pulse => new_received_data_read_pulse,
      uart_data_in                         => data_in,
      uart_ready_to_trasmit                => ready_to_trasmit,
      to_uart_new_data_to_trasmit          => new_data_to_trasmit
      );



end struct;

---*******************************************************---
---*******************************************************---
---************  WB_MASTER *******************************---
---*******************************************************---
---*******************************************************---
-----------------------------------------------------------
----- package PK_WB_MASTER
-----------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

package PK_WB_MASTER is
  component WB_MASTER is
    port(
      CLK_I   : in std_logic;
      RESET_I : in std_logic;

      -- WB Interface
      CYC_O : out std_logic;
      CTI_O : out std_logic_vector(2 downto 0);
      STB_O : out std_logic;
      ACK_I : in  std_logic;
      WE_O  : out std_logic;
      ADR_O : out std_logic_vector(7 downto 0);
      DAT_I : in  std_logic_vector(7 downto 0);
      DAT_O : out std_logic_vector(7 downto 0);

      -- Uart interface / receive data
      uart_data_out                        : in  std_logic_vector(7 downto 0);
      uart_new_received_data               : in  std_logic;
      to_uart_new_received_data_read_pulse : out std_logic;

      -- Uart interface / receive data
      uart_data_in                : out std_logic_vector(7 downto 0);
      uart_ready_to_trasmit       : in  std_logic;
      to_uart_new_data_to_trasmit : out std_logic
      );
  end component;
end package;

-----------------------------------------------------------
----- ENTITY WB_MASTER
-----------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity wb_master is
  port(
    CLK_I   : in std_logic;
    RESET_I : in std_logic;

    -- WB Interface
    CYC_O : out std_logic;
    CTI_O : out std_logic_vector(2 downto 0);
    STB_O : out std_logic;
    ACK_I : in  std_logic;
    WE_O  : out std_logic;
    ADR_O : out std_logic_vector(7 downto 0);
    DAT_I : in  std_logic_vector(7 downto 0);
    DAT_O : out std_logic_vector(7 downto 0);

    -- Uart interface / receive data
    uart_data_out                        : in  std_logic_vector(7 downto 0);
    uart_new_received_data               : in  std_logic;
    to_uart_new_received_data_read_pulse : out std_logic;

    -- Uart interface / receive data
    uart_data_in                : out std_logic_vector(7 downto 0);
    uart_ready_to_trasmit       : in  std_logic;
    to_uart_new_data_to_trasmit : out std_logic
    );
end wb_master;


architecture behav of wb_master is

------------------------- CONSTANT ----------------------------------           

------------------------- signals ----------------------------------
-- address_register
  signal address         : std_logic_vector(7 downto 0);
  signal address_reg_ena : std_logic;

-- data_from_uart_register
  signal data_from_uart         : std_logic_vector(7 downto 0);
  signal data_from_uart_reg_ena : std_logic;

-- data_to_uart_register
  signal data_to_uart         : std_logic_vector(7 downto 0);
  signal data_to_uart_reg_ena : std_logic;

-- states
  type   state_type is (wb_idle, wb_deco_cmd, wb_set_address, wb_write, wb_read, wb_data_to_uart, wb_wait_data_to_write, wb_read_data);
  signal wb_state, wb_nxt_state : state_type;

begin
---*******
  address_register :
---*******
  process(CLK_I, RESET_I)
  begin
    if (CLK_I'event and CLK_I = '1') then
      if (address_reg_ena = '1') then
        address <= uart_data_out;
      end if;
    end if;
  end process;

  ADR_O <= address;

---*******
  data_from_uart_register :
---*******
  process(CLK_I, RESET_I)
  begin
    if (CLK_I'event and CLK_I = '1') then
      if (data_from_uart_reg_ena = '1') then
        data_from_uart <= uart_data_out;
      end if;
    end if;
  end process;

  DAT_O <= data_from_uart;

---*******
  data_to_uart_register :
---*******
  process(CLK_I, RESET_I)
  begin
    if (CLK_I'event and CLK_I = '1') then
      if (data_to_uart_reg_ena = '1') then
        data_to_uart <= DAT_I;
      end if;
    end if;
  end process;

  uart_data_in <= data_to_uart;


---*******
  wb_control_logic :
---*******
  process(CLK_I, RESET_I, wb_state, ACK_I, uart_new_received_data, uart_ready_to_trasmit, uart_data_out)
  begin
    case wb_state is
      when wb_idle =>
        CYC_O <= '0';
        CTI_O <= "000";
        STB_O <= '0';
        WE_O  <= '0';

        to_uart_new_received_data_read_pulse <= '0';
        to_uart_new_data_to_trasmit          <= '0';

        address_reg_ena        <= '0';
        data_from_uart_reg_ena <= '0';
        data_to_uart_reg_ena   <= '0';

        if (uart_new_received_data = '1') then
          wb_nxt_state <= wb_deco_cmd;
        else
          wb_nxt_state <= wb_idle;
        end if;
        
      when wb_deco_cmd =>
        CYC_O <= '0';
        CTI_O <= "000";
        STB_O <= '0';
        WE_O  <= '0';

        to_uart_new_received_data_read_pulse <= '1';
        to_uart_new_data_to_trasmit          <= '0';

        address_reg_ena        <= '0';
        data_from_uart_reg_ena <= '0';
        data_to_uart_reg_ena   <= '0';

        if (uart_data_out = "00000001") then
          wb_nxt_state <= wb_set_address;
        elsif (uart_data_out = "00000010") then
          wb_nxt_state <= wb_wait_data_to_write;
        elsif (uart_data_out = "00000100") then
          wb_nxt_state <= wb_read;
        else
          wb_nxt_state <= wb_idle;
        end if;
        
      when wb_set_address =>
        CYC_O <= '0';
        CTI_O <= "000";
        STB_O <= '0';
        WE_O  <= '0';

        to_uart_new_data_to_trasmit <= '0';

        data_from_uart_reg_ena <= '0';
        data_to_uart_reg_ena   <= '0';

        if (uart_new_received_data = '1') then
          wb_nxt_state                         <= wb_idle;
          to_uart_new_received_data_read_pulse <= '1';
          address_reg_ena                      <= '1';
        else
          wb_nxt_state                         <= wb_set_address;
          to_uart_new_received_data_read_pulse <= '0';
          address_reg_ena                      <= '0';
        end if;
        
      when wb_wait_data_to_write =>
        CYC_O <= '0';
        CTI_O <= "000";
        STB_O <= '0';
        WE_O  <= '0';

        to_uart_new_data_to_trasmit <= '0';

        address_reg_ena      <= '0';
        data_to_uart_reg_ena <= '0';

        if (uart_new_received_data = '1') then
          wb_nxt_state                         <= wb_write;
          to_uart_new_received_data_read_pulse <= '1';
          data_from_uart_reg_ena               <= '1';
        else
          wb_nxt_state                         <= wb_wait_data_to_write;
          to_uart_new_received_data_read_pulse <= '0';
          data_from_uart_reg_ena               <= '0';
        end if;
        
      when wb_write =>
        CYC_O <= '1';
        CTI_O <= "111";
        STB_O <= '1';
        WE_O  <= '1';

        to_uart_new_data_to_trasmit          <= '0';
        to_uart_new_received_data_read_pulse <= '0';

        address_reg_ena        <= '0';
        data_to_uart_reg_ena   <= '0';
        data_from_uart_reg_ena <= '0';

        if (ACK_I = '1') then
          wb_nxt_state <= wb_idle;
        else
          wb_nxt_state <= wb_write;
        end if;
        
      when wb_read =>
        CYC_O <= '1';
        CTI_O <= "111";
        STB_O <= '0';
        WE_O  <= '0';

        to_uart_new_data_to_trasmit          <= '0';
        to_uart_new_received_data_read_pulse <= '0';

        address_reg_ena        <= '0';
        data_to_uart_reg_ena   <= '0';
        data_from_uart_reg_ena <= '0';

        wb_nxt_state <= wb_read_data;
        
      when wb_read_data =>
        CYC_O <= '1';
        CTI_O <= "111";
        STB_O <= '1';
        WE_O  <= '0';

        to_uart_new_data_to_trasmit          <= '0';
        to_uart_new_received_data_read_pulse <= '0';

        address_reg_ena        <= '0';
        data_to_uart_reg_ena   <= '1';
        data_from_uart_reg_ena <= '0';

        if (ACK_I = '1') then
          wb_nxt_state <= wb_data_to_uart;
        else
          wb_nxt_state <= wb_read_data;
        end if;
        
      when wb_data_to_uart =>
        CYC_O <= '0';
        CTI_O <= "000";
        STB_O <= '0';
        WE_O  <= '0';

        to_uart_new_received_data_read_pulse <= '0';

        address_reg_ena        <= '0';
        data_to_uart_reg_ena   <= '0';
        data_from_uart_reg_ena <= '0';

        if (uart_ready_to_trasmit = '1') then
          wb_nxt_state                <= wb_idle;
          to_uart_new_data_to_trasmit <= '1';
        else
          wb_nxt_state                <= wb_data_to_uart;
          to_uart_new_data_to_trasmit <= '0';
        end if;
        
        
      when others =>
        CYC_O <= '0';
        CTI_O <= "000";
        STB_O <= '0';
        WE_O  <= '0';

        to_uart_new_received_data_read_pulse <= '0';
        to_uart_new_data_to_trasmit          <= '0';

        address_reg_ena        <= '0';
        data_from_uart_reg_ena <= '0';
        data_to_uart_reg_ena   <= '0';

        wb_nxt_state <= wb_idle;
        
    end case;
  end process;


  sync : process (CLK_I, RESET_I)
  begin
    if RESET_I = '1' then
      wb_state <= wb_idle;
    elsif CLK_I'event and CLK_I = '1' then
      wb_state <= wb_nxt_state;
    end if;
  end process;
  
  
end behav;

---*******************************************************---
---*******************************************************---
---************  UART      *******************************---
---*******************************************************---
---*******************************************************---

-----------------------------------------------------------
----- package PK_UART
-----------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

package PK_UART is
  component UART is
    generic(
      BAUD_RATE_16x_COUNTER_REFERENCE : integer
      ); 
    port(
      CLK_I   : in  std_logic;
      RESET_I : in  std_logic;
      --- RS232 signals
      TX_O    : out std_logic;
      RX_I    : in  std_logic;

      --- reception signals interface
      data_out                     : out std_logic_vector(7 downto 0);
      new_received_data            : out std_logic;
      new_received_data_read_pulse : in  std_logic;

      --- trasmition signals interface
      data_in             : in  std_logic_vector(7 downto 0);
      ready_to_trasmit    : out std_logic;
      new_data_to_trasmit : in  std_logic
      );
  end component;
end package;

-----------------------------------------------------------
----- ENTITY uart
-----------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.math_real.all;
use ieee.std_logic_unsigned.all;
library work;
use work.PK_UART_RX.all;
use work.PK_UART_TX.all;


entity uart is
  generic(
    BAUD_RATE_16x_COUNTER_REFERENCE : integer
    ); 
  port(
    CLK_I   : in  std_logic;
    RESET_I : in  std_logic;
    --- RS232 signals
    TX_O    : out std_logic;
    RX_I    : in  std_logic;

    --- reception signals interface
    data_out                     : out std_logic_vector(7 downto 0);
    new_received_data            : out std_logic;
    new_received_data_read_pulse : in  std_logic;

    --- trasmition signals interface
    data_in             : in  std_logic_vector(7 downto 0);
    ready_to_trasmit    : out std_logic;
    new_data_to_trasmit : in  std_logic
    );
end uart;


architecture struct of uart is

------------------------- constant ---------------------------------
  constant BAUD_RATE_16x_COUNTER_WIDTH : integer := integer(CEIL(LOG(real(BAUD_RATE_16x_COUNTER_REFERENCE+1), 2.0)));

------------------------- signals ----------------------------------
-- Baud Rate Generators
  signal baud_rate_16x_counter     : std_logic_vector(BAUD_RATE_16x_COUNTER_WIDTH-1 downto 0);
  signal baud_rate_16x_counter_ref : std_logic_vector(BAUD_RATE_16x_COUNTER_WIDTH-1 downto 0);
  signal baud_rate_16x_pulse       : std_logic;

--------------------------------------------------------------------
begin


  baud_rate_16x_counter_ref <= CONV_STD_LOGIC_VECTOR(BAUD_RATE_16x_COUNTER_REFERENCE, BAUD_RATE_16x_COUNTER_WIDTH);

---********
  baud_rate_16x_generation :
---******** 
  process(CLK_I, RESET_I)
  begin
    if (CLK_I'event and CLK_I = '1') then
      if RESET_I = '1' then
        baud_rate_16x_counter <= (others => '0');
        baud_rate_16x_pulse   <= '0';
      elsif (baud_rate_16x_counter = baud_rate_16x_counter_ref) then
        baud_rate_16x_pulse   <= '1';
        baud_rate_16x_counter <= (others => '0');
      else
        baud_rate_16x_counter <= baud_rate_16x_counter + 1;
        baud_rate_16x_pulse   <= '0';
      end if;
    end if;
  end process;



  inst_uart_rx : UART_RX
    port map(
      CLK_I   => CLK_I,
      RESET_I => RESET_I,

      tbit_div_pulse => baud_rate_16x_pulse,

      serial_in => RX_I,

      -- To aplication
      data_out                     => data_out,
      new_received_data            => new_received_data,
      new_received_data_read_pulse => new_received_data_read_pulse
      );

  inst_uart_tx : UART_TX
    port map(
      CLK_I   => CLK_I,
      RESET_I => RESET_I,

      tbit_div_pulse => baud_rate_16x_pulse,

      serial_out => TX_O,

      -- To aplication
      data_in             => data_in,
      ready_to_trasmit    => ready_to_trasmit,
      new_data_to_trasmit => new_data_to_trasmit
      );   

end struct;

---*******************************************************---
---*******************************************************---
---************  UART_RX   *******************************---
---*******************************************************---
---*******************************************************---
-----------------------------------------------------------
----- package PK_UART_TX
-----------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

package PK_UART_RX is
  component UART_RX is
    port(
      CLK_I   : in std_logic;
      RESET_I : in std_logic;

      tbit_div_pulse : in std_logic;

      -- From RS232
      serial_in : in std_logic;

      -- To aplication
      data_out                     : out std_logic_vector(7 downto 0);
      new_received_data            : out std_logic;
      new_received_data_read_pulse : in  std_logic
      );
  end component;
end package;

-----------------------------------------------------------
----- ENTITY uart_rx
-----------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


entity uart_rx is
  port(
    CLK_I   : in std_logic;
    RESET_I : in std_logic;

    tbit_div_pulse : in std_logic;

    -- From RS232
    serial_in : in std_logic;

    -- To aplication
    data_out                     : out std_logic_vector(7 downto 0);
    new_received_data            : out std_logic;
    new_received_data_read_pulse : in  std_logic
    );
end uart_rx;


architecture behav of uart_rx is

------------------------- CONSTANT ----------------------------------


------------------------- signals ----------------------------------
-- RX shift registers
  signal rx_shift_register : std_logic_vector(7 downto 0);  -- 8 data bits

-- tbit_div16_pulse counter
  signal tbit_div_pulse_count : std_logic_vector(3 downto 0);
  signal tbit_div_pulse_clear : std_logic;

-- readbit_counter
  signal read_bit_count : std_logic_vector(3 downto 0);


-- general control signals 
  signal read_bit : std_logic;
  signal init_all : std_logic;

-- new data notification
  signal new_received_data_set : std_logic;

-- states
  type state_type is
    (rx_idle, rx_check_start_bit, rx_receive, rx_finish);
  signal rx_state, rx_nxt_state : state_type;

--------------------------------------------------------------------

begin

---********
  readbit_counter :
---********
  process(CLK_I, RESET_I)
  begin
    if (CLK_I'event and CLK_I = '1') then
      if (RESET_I = '1' or init_all = '1') then
        read_bit_count <= (others => '0');
      elsif (read_bit = '1') then
        read_bit_count <= read_bit_count + 1;
      end if;
    end if;
  end process;




---********
-- tbit_div_pulse counter:
---********
  process(CLK_I, RESET_I)
  begin
    if (CLK_I'event and CLK_I = '1') then
      if ((RESET_I = '1') or (init_all = '1') or (tbit_div_pulse_clear = '1')) then
        tbit_div_pulse_count <= (others => '0');
      elsif (tbit_div_pulse = '1') then
        tbit_div_pulse_count <= tbit_div_pulse_count + 1;
      end if;
    end if;
  end process;

---********
-- new_data_notification:
---********
  process(CLK_I, RESET_I)
  begin
    if (CLK_I'event and CLK_I = '1') then
      if (RESET_I = '1'or new_received_data_read_pulse = '1') then
        new_received_data <= '0';
      elsif (new_received_data_set = '1') then
        new_received_data <= '1';
      end if;
    end if;
  end process;

---********
  rx_shift_register_generation :
---********
  process(CLK_I, RESET_I)
  begin
    if (CLK_I'event and CLK_I = '1') then
      if (read_bit = '1') then
        rx_shift_register(6 downto 0) <= rx_shift_register(7 downto 1);
        rx_shift_register(7)          <= serial_in;
      end if;
    end if;
  end process;

  data_out <= rx_shift_register;

---*******
  rx_control_logic :
---*******
  process(CLK_I, RESET_I, rx_state, serial_in, tbit_div_pulse_count, read_bit_count)
  begin
    case rx_state is
      when rx_idle =>
        new_received_data_set <= '0';
        tbit_div_pulse_clear  <= '1';

        init_all <= '1';
        read_bit <= '0';

        if (serial_in = '0') then
          rx_nxt_state <= rx_check_start_bit;
        else
          rx_nxt_state <= rx_idle;
        end if;
        
      when rx_check_start_bit =>
        new_received_data_set <= '0';
        init_all              <= '0';
        read_bit              <= '0';
        if (tbit_div_pulse_count = 7) then
          tbit_div_pulse_clear <= '1';
          if (serial_in = '0') then
            rx_nxt_state <= rx_receive;
          else
            rx_nxt_state <= rx_idle;
          end if;
        else
          tbit_div_pulse_clear <= '0';
          rx_nxt_state         <= rx_check_start_bit;
        end if;


      when rx_receive =>
        init_all <= '0';
        if (tbit_div_pulse_count = 15) then
          read_bit             <= '1';
          tbit_div_pulse_clear <= '1';
        else
          read_bit             <= '0';
          tbit_div_pulse_clear <= '0';
        end if;
        if (read_bit_count = 8) then
          rx_nxt_state          <= rx_finish;
          new_received_data_set <= '1';
        else
          rx_nxt_state          <= rx_receive;
          new_received_data_set <= '0';
        end if;
        
      when rx_finish =>
        new_received_data_set <= '0';
        init_all              <= '0';
        read_bit              <= '0';
        if (tbit_div_pulse_count = 10) then
          tbit_div_pulse_clear <= '1';
          rx_nxt_state         <= rx_idle;
        else
          tbit_div_pulse_clear <= '0';
          rx_nxt_state         <= rx_finish;
        end if;
        
      when others =>
        new_received_data_set <= '0';
        tbit_div_pulse_clear  <= '0';
        init_all              <= '0';
        read_bit              <= '0';
        rx_nxt_state          <= rx_idle;
        
    end case;
  end process;


  sync : process (CLK_I, RESET_I)
  begin
    if RESET_I = '1' then
      rx_state <= rx_idle;
    elsif CLK_I'event and CLK_I = '1' then
      rx_state <= rx_nxt_state;
    end if;
  end process;

end behav;

---*******************************************************---
---*******************************************************---
---************  UART_TX   *******************************---
---*******************************************************---
---*******************************************************---
-----------------------------------------------------------
----- package PK_UART_TX
-----------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

package PK_UART_TX is
  component UART_TX is
    port(

      CLK_I   : in std_logic;
      RESET_I : in std_logic;

      tbit_div_pulse : in  std_logic;
      -- To RS232
      serial_out     : out std_logic;

      -- To aplication
      data_in             : in  std_logic_vector(7 downto 0);
      ready_to_trasmit    : out std_logic;
      new_data_to_trasmit : in  std_logic
      );
  end component;
end package;

-----------------------------------------------------------
----- ENTITY uart_tx
-----------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


entity uart_tx is
  port(
    CLK_I   : in std_logic;
    RESET_I : in std_logic;

    tbit_div_pulse : in  std_logic;
    -- To RS232
    serial_out     : out std_logic;

    -- To aplication
    data_in             : in  std_logic_vector(7 downto 0);
    ready_to_trasmit    : out std_logic;
    new_data_to_trasmit : in  std_logic
    );
end uart_tx;


architecture behav of uart_tx is

------------------------- CONSTANT ----------------------------------


------------------------- signals ----------------------------------
-- TX shift registers
  signal tx_shift_register      : std_logic_vector(9 downto 0);  -- start bit + 8 data bits + stop bit
  signal tx_shift_register_load : std_logic;

-- tbit_div16_pulse counter
  signal tbit_div_pulse_count : std_logic_vector(3 downto 0);
  signal tbit_div_pulse_clear : std_logic;


-- sentbit_counter
  signal sent_bit_count : std_logic_vector(3 downto 0);

-- general control signals 
  signal send_bit : std_logic;
  signal init_all : std_logic;

-- states
  type state_type is
    (tx_idle, tx_prepare_trasmit, tx_trasmit);
  signal tx_state, tx_nxt_state : state_type;

--------------------------------------------------------------------

begin

---********
  sentbit_counter :
---********
  process(CLK_I, RESET_I)
  begin
    if (CLK_I'event and CLK_I = '1') then
      if (RESET_I = '1' or init_all = '1') then
        sent_bit_count <= (others => '0');
      elsif (send_bit = '1') then
        sent_bit_count <= sent_bit_count + 1;
      end if;
    end if;
  end process;




---********
-- tbit_div_pulse counter:
---********
  process(CLK_I, RESET_I)
  begin
    if (CLK_I'event and CLK_I = '1') then
      if ((RESET_I = '1') or (init_all = '1') or (tbit_div_pulse_clear = '1')) then
        tbit_div_pulse_count <= (others => '0');
      elsif (tbit_div_pulse = '1') then
        tbit_div_pulse_count <= tbit_div_pulse_count + 1;
      end if;
    end if;
  end process;

---********
  tx_shift_register_generation :
---********
  process(CLK_I, RESET_I)
  begin
    if (CLK_I'event and CLK_I = '1') then
      if (RESET_I = '1' or init_all = '1') then
        tx_shift_register <= (others => '1');
      elsif (tx_shift_register_load = '1') then
        tx_shift_register(0)          <= '0';
        tx_shift_register(8 downto 1) <= data_in;
      elsif (send_bit = '1') then
        tx_shift_register(7 downto 0) <= tx_shift_register(8 downto 1);
        tx_shift_register(8)          <= '1';
      end if;
    end if;
  end process;



---*******
  tx_control_logic :
---*******
  process(CLK_I, RESET_I, tx_state, new_data_to_trasmit, tbit_div_pulse_count, sent_bit_count)
  begin
    case tx_state is
      when tx_idle =>
        ready_to_trasmit       <= '1';
        init_all               <= '1';
        tbit_div_pulse_clear   <= '1';
        send_bit               <= '0';
        tx_shift_register_load <= '0';

        if (new_data_to_trasmit = '1') then
          tx_nxt_state <= tx_prepare_trasmit;
        else
          tx_nxt_state <= tx_idle;
        end if;
        
      when tx_prepare_trasmit =>
        ready_to_trasmit       <= '0';
        init_all               <= '0';
        send_bit               <= '0';
        tbit_div_pulse_clear   <= '0';
        tx_shift_register_load <= '1';

        tx_nxt_state <= tx_trasmit;
        
      when tx_trasmit =>
        ready_to_trasmit       <= '0';
        init_all               <= '0';
        tx_shift_register_load <= '0';

        if (tbit_div_pulse_count = 15) then
          send_bit             <= '1';
          tbit_div_pulse_clear <= '1';
        else
          send_bit             <= '0';
          tbit_div_pulse_clear <= '0';
        end if;
        if (sent_bit_count = 10) then
          tx_nxt_state <= tx_idle;
        else
          tx_nxt_state <= tx_trasmit;
        end if;
        
      when others =>
        tbit_div_pulse_clear   <= '0';
        ready_to_trasmit       <= '0';
        init_all               <= '1';
        tx_shift_register_load <= '0';
        send_bit               <= '0';
        tx_nxt_state           <= tx_idle;
    end case;
  end process;


  serial_out <= tx_shift_register(0);

  sync : process (CLK_I, RESET_I)
  begin
    if RESET_I = '1' then
      tx_state <= tx_idle;
    elsif CLK_I'event and CLK_I = '1' then
      tx_state <= tx_nxt_state;
    end if;
  end process;

end behav;


