--------------------------------------------------------------------
-- Project : Serie to WB master reference design
-- Author : Sebastin Fernndez
-- Date : 27/08/2008
-- File : serie2wbm_ref_design.vhd
-- Design : 
--------------------------------------------------------------------
-- Descripcin : 
-- Diseo de referencia para placa UP1 (chip EPF10K20RC240-4) con: 
--    * bloque serie2wbm 
--    * bloque wb_register 
--    * multiplexor conectado a bloques hex27seg y estos display 7 seg que permite visualizar
--       * contenido de wb_register (MUX_SEL = 1)
--       * bus de direcciones de serie2wbm (MUX_SEL = 0)
-- 
-- Asignacin de pines sugerida:  
--   * CLK_I : pin 91
--   * RESET_In :  pin 41 (FLEX_SWITCH-1)
--   * TX_O : pin 101 (FLEX_EXPAN_A hole 60)
--   * RX_I : pin 99  (FLEX_EXPAN_A hole 58)
--   * DIG1(6): pin 6 (Digit 1, Segment a)
--   * DIG1(5): pin 7 (Digit 1, Segment b)
--   * DIG1(4): pin 8 (Digit 1, Segment c)
--   * DIG1(3): pin 9 (Digit 1, Segment d)
--   * DIG1(2): pin 11 (Digit 1, Segment e)
--   * DIG1(1): pin 12 (Digit 1, Segment f)
--   * DIG1(0): pin 13 (Digit 1, Segment g)
--   * DIG2(6): pin 17 (Digit 1, Segment a)
--   * DIG2(5): pin 18 (Digit 1, Segment b)
--   * DIG2(4): pin 19 (Digit 1, Segment c)
--   * DIG2(3): pin 20 (Digit 1, Segment d)
--   * DIG2(2): pin 21 (Digit 1, Segment e)
--   * DIG2(1): pin 23 (Digit 1, Segment f)
--   * DIG2(0): pin 24 (Digit 1, Segment g)
--   * MUX_SEL:  pin 40 (FLEX_SWITCH-2)
--
-- Valor de Generic BAUD_RATE_16x_COUNTER_REFERENCE en serie2wbm 
--    * para FCK=25.175 MHz y trasmisin serie 9600 bps: 174 
--    * para simulacin: FCK= 10MHz y trasmisin serie 312500 bps: BAUD_RATE_16x_COUNTER_REFERENCE = 2
--       * Tck = 100ns y Tbit= 4.5us
--------------------------------------------------------------------
-- $Log$
--------------------------------------------------------------------


-----------------------------------------------------------
----- ENTITY serie2wbm_ref_design
-----------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
library work;
use work.PK_WB_REGISTER.all;
use work.PK_SERIE2WBM.all;
use work.PK_hex27seg.all;

entity serie2wbm_ref_design is
   port(
      CLK_I                : in std_logic;
      RESET_In             : in std_logic;  -- Active Low
   	--- RS232 signals
      TX_O                  : out std_logic;
   	RX_I                  : in std_logic;
   	
   	MUX_SEL               : in std_logic; 		
   	-- 7Segments outputs
      DIG1 			   : out std_logic_vector(6 downto 0); 
      DIG2 			   : out std_logic_vector(6 downto 0) 
      );
end serie2wbm_ref_design;


architecture struct of serie2wbm_ref_design is

---- Signals ----
-- general signals
signal RESET_I  		: std_logic; 
-- WB Interface signals
signal wb_CYC_O 		: std_logic;                   
signal wb_CTI_O 		: std_logic_vector(2 downto 0);
signal wb_STB_O 		: std_logic;                   
signal wb_ACK_I 		: std_logic;                    
signal wb_WE_O 		: std_logic;                   
signal wb_ADR_O 		: std_logic_vector(7 downto 0);
signal wb_DAT_I 		: std_logic_vector(7 downto 0); 
signal wb_DAT_O 		: std_logic_vector(7 downto 0);
-- wb_register related signals 
signal Q_WB_REGISTER : std_logic_vector(7 downto 0); 
-- MUX related signals
signal mux_out       : std_logic_vector(7 downto 0); 
   		
begin

RESET_I <= not(RESET_In);

serie2wb_instance: serie2wbm
      generic map( 
         BAUD_RATE_16x_COUNTER_REFERENCE => 2
        )
      port map(
   	   CLK_I  => CLK_I,
   	   RESET_I => RESET_I,
   	   --- RS232 signals
         TX_O   => TX_O,
   		RX_I   => RX_I,
   		
   		-- WB Interface
         CYC_O  => wb_CYC_O,
         CTI_O  => wb_CTI_O,
         STB_O  => wb_STB_O,
	     	ACK_I  => wb_ACK_I,
	     	WE_O 	 => wb_WE_O,
	     	ADR_O  => wb_ADR_O,
	     	DAT_I  => wb_DAT_I,  	  
         DAT_O  => wb_DAT_O
         );

wb_register_instance: wb_register
   port map(
      RST_I   => RESET_I,
      CLK_I   => CLK_I,
      DAT_I   => wb_DAT_O,
      DAT_O   => wb_DAT_I,
      ACK_O   => wb_ACK_I,
      CYC_I   => wb_CYC_O,
      STB_I   => wb_STB_O,
      CTI_I   => wb_CTI_O,
      WE_I    => wb_WE_O,
      --register output signals
      Q       => Q_WB_REGISTER
   ); 


mux_out <= Q_WB_REGISTER when MUX_SEL = '1' else wb_ADR_O;

inst_hex27seg1: hex27seg
      port map(
   	   hex_in  => mux_out(3 downto 0),
   	   seg_out => DIG2
         );

inst_hex27seg2: hex27seg
      port map(
   	   hex_in  => mux_out(7 downto 4),
   	   seg_out => DIG1
         );
         
end struct;

