
------------------------------------------------------------------
---- Package: wb_register
------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use IEEE.std_logic_unsigned.all;
package pk_wb_register is
   component wb_register IS
   GENERIC (data_width: POSITIVE := 8);
   PORT(
      --WB signals
      RST_I   : IN  STD_LOGIC;
      CLK_I   : IN  STD_LOGIC;
      DAT_I   : IN STD_LOGIC_VECTOR(data_width-1 downto 0);
      DAT_O   : OUT  STD_LOGIC_VECTOR(data_width-1 downto 0);
      ACK_O   : OUT STD_LOGIC;
      CYC_I   : IN  STD_LOGIC;
      STB_I   : IN  STD_LOGIC;
      CTI_I   : IN  STD_LOGIC_VECTOR(2 downto 0);
      WE_I    : IN  STD_LOGIC;
      --register output signals
      Q        : OUT  STD_LOGIC_VECTOR(data_width-1 downto 0)
    );
   END component;
end package;   
      
      
------------------------------------------------------------------
---- Entity: wb_register
------------------------------------------------------------------      
LIBRARY ieee ;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
LIBRARY lpm;
USE lpm.lpm_components.ALL;

-----------------------------
ENTITY wb_register IS
   GENERIC (data_width: POSITIVE := 32);
   PORT(
      --WB signals
      RST_I   : IN  STD_LOGIC;
      CLK_I   : IN  STD_LOGIC;
      DAT_I    : IN STD_LOGIC_VECTOR(data_width-1 downto 0);
      DAT_O    : OUT  STD_LOGIC_VECTOR(data_width-1 downto 0);
      ACK_O   : OUT STD_LOGIC;
      CYC_I   : IN  STD_LOGIC;
      STB_I   : IN  STD_LOGIC;
      CTI_I   : IN  STD_LOGIC_VECTOR(2 downto 0);
      WE_I    : IN  STD_LOGIC;
      --register output signals
      Q        : OUT  STD_LOGIC_VECTOR(data_width-1 downto 0)
      );
END wb_register;
-----------------------------

ARCHITECTURE synth OF wb_register IS

signal q_register : STD_LOGIC_VECTOR(data_width-1 downto 0);
signal r_ACK    : STD_LOGIC;

-- states
  type   state_type is (waiting_cyc, ack_active, ack_inactive);
  signal ack_state : state_type;

BEGIN

----------------------------------------
--- register generation
----------------------------------------
register_process: process (RST_I, CLK_I)
   begin
   if CLK_I'event and CLK_I='1' then 
      if RST_I='1' then
           q_register <= (OTHERS => '0');
      elsif (CYC_I='1' and STB_I='1' and WE_I='1' and r_ACK='1') then 
         q_register <= DAT_I;
      end if;
   end if;
 end process;

   Q <= q_register;


-----------------------
-- registrered ACK generation
-----------------------
ack_gen_process: process (RST_I, CLK_I, ack_state)
   begin
   if CLK_I'event and CLK_I='1' then 
      if RST_I='1' then
         r_ACK <= '0';
         ack_state <= waiting_cyc;
      else
         case ack_state is
           when waiting_cyc =>
             -- el registro siempre esta listo para recibir datos, por lo que
             -- en caso de detectar un comienzo de ciclo, debe avisar (ack=1)
             -- que esta listo para recibir/entregar un dato 
             if (CYC_I = '0') then
               ack_state <= waiting_cyc;
               r_ACK     <= '0';
             else
                 ack_state <= ack_active;
                 r_ACK     <= '1';
             end if;

          when ack_active =>
             -- si por alguna razn CYC=0 desactivo ACK y vuelvo esperar un
             -- comienzo de ciclo           
             if (CYC_I = '0') then          
               ack_state <= waiting_cyc;                                     
               r_ACK     <= '0';

             else
               if (CTI_I="001" or CTI_I="010") then
                 -- si es un ciclo burst, mantengo ACK=1
                 ack_state <= ack_active;  
                 r_ACK     <= '1';
               elsif (CTI_I="111") then
                 if (STB_I = '1') then
                   -- si es la ltima transferencia del ciclo y STB='1'
                   -- voy esperar un nuevo ciclo
                   ack_state <= waiting_cyc;
                   r_ACK     <= '0';
                 else
                   -- si STB='0' mantengo ACK=1
                   ack_state <= ack_active;
                   r_ACK     <= '1';
                 end if;
               elsif (CTI_I = "000") then
                 if (STB_I = '1') then
                   -- si es un ciclo clsico y STB='1'
                   -- doy ACK=0 en el ciclo que viene
                   ack_state <= ack_inactive;
                   r_ACK     <= '0';
                 else
                    -- si STB='0' mantengo ACK=1
                   ack_state <= ack_active;
                   r_ACK     <= '1';
                 end if;
               else
                 -- para otro valor de CTI mantengo ACK =1
                   ack_state <= ack_active;
                   r_ACK     <= '1';
               end if;              
			end if;
         when ack_inactive =>
             -- si por CYC=0 vuelvo a esperar un
             -- comienzo de ciclo           
             if (CYC_I = '0') then          
               ack_state <= waiting_cyc;                                     
               r_ACK     <= '0';
             else
               ack_state <= ack_active;  
               r_ACK     <= '1';
             end if;
             
         end case;
     end if;
   end if;           
 end process;

---------------------
-- WB signals
---------------------
   DAT_O <= q_register;
   ACK_O <= r_ACK;   
    
end synth;

